@@ -1,6 +1,6 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/__init__.py
-__version__=''' $Id: __init__.py 2385 2004-06-17 15:26:05Z rgbecker $ '''
-__doc__="""
-"""
\ No newline at end of file
+__version__=''' $Id$ '''
+__doc__="""Internal functionality for creating PDF files - not part of API
+"""
