@@ -1,15 +1,15 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/figures.py
 """This includes some demos of platypus for use in the API proposal"""
-__version__=''' $Id: figures.py 2969 2006-09-07 14:57:53Z rgbecker $ '''
+__version__=''' $Id$ '''
 
 import os
 
 from reportlab.lib import colors
 from reportlab.pdfgen.canvas import Canvas
 from reportlab.lib.styles import ParagraphStyle
-from reportlab.lib.utils import recursiveImport
+from reportlab.lib.utils import recursiveImport, strTypes
 from reportlab.platypus import Frame
 from reportlab.platypus import Flowable
 from reportlab.platypus import Paragraph
@@ -17,19 +17,23 @@ from reportlab.lib.units import inch
 from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER
 from reportlab.lib.validators import isColor
 from reportlab.lib.colors import toColor
+from reportlab.lib.styles import _baseFontName, _baseFontNameI
 
-captionStyle = ParagraphStyle('Caption', fontName='Times-Italic', fontSize=10, alignment=TA_CENTER)
+captionStyle = ParagraphStyle('Caption', fontName=_baseFontNameI, fontSize=10, alignment=TA_CENTER)
 
 class Figure(Flowable):
     def __init__(self, width, height, caption="",
-                 captionFont="Times-Italic", captionSize=12,
+                 captionFont=_baseFontNameI, captionSize=12,
                  background=None,
                  captionTextColor=toColor('black'),
                  captionBackColor=None,
-                 border=1,
+                 border=None,
                  spaceBefore=12,
                  spaceAfter=12,
                  captionGap=None,
+                 captionAlign='centre',
+                 captionPosition='bottom',
+                 hAlign='CENTER',
                  ):
         Flowable.__init__(self)
         self.width = width
@@ -39,13 +43,17 @@ class Figure(Flowable):
         self.captionSize = captionSize
         self.captionTextColor = captionTextColor
         self.captionBackColor = captionBackColor
-        self.captionGap = captionGap
+        self.captionGap = captionGap or 0.5*captionSize
+        self.captionAlign = captionAlign
+        self.captionPosition = captionPosition
         self._captionData = None
         self.captionHeight = 0  # work out later
         self.background = background
         self.border = border
         self.spaceBefore = spaceBefore
         self.spaceAfter = spaceAfter
+        self.hAlign=hAlign
+        self._getCaptionPara()  #Larry Meyn's fix - otherwise they all get the number of the last chapter.
 
     def _getCaptionPara(self):
         caption = self.caption
@@ -53,47 +61,81 @@ class Figure(Flowable):
         captionSize = self.captionSize
         captionTextColor = self.captionTextColor
         captionBackColor = self.captionBackColor
-        if self._captionData!=(caption,captionFont,captionSize,captionTextColor,captionBackColor):
-            self._captionData = (caption,captionFont,captionSize,captionTextColor,captionBackColor)
-            self.captionStyle = ParagraphStyle(
-                'Caption',
-                fontName=captionFont,
-                fontSize=captionSize,
-                leading=1.2*captionSize,
-                textColor = captionTextColor,
-                backColor = captionBackColor,
-                spaceBefore=self.captionGap or 0.5*captionSize,
-                alignment=TA_CENTER)
-            #must build paragraph now to get sequencing in synch with rest of story
-            self.captionPara = Paragraph(self.caption, self.captionStyle)
+        captionAlign = self.captionAlign
+        captionPosition = self.captionPosition
+        if self._captionData!=(caption,captionFont,captionSize,captionTextColor,captionBackColor,captionAlign,captionPosition):
+            self._captionData = (caption,captionFont,captionSize,captionTextColor,captionBackColor,captionAlign,captionPosition)
+            if isinstance(caption,Paragraph):
+                self.captionPara = caption
+            elif isinstance(caption,strTypes):
+                self.captionStyle = ParagraphStyle(
+                    'Caption',
+                    fontName=captionFont,
+                    fontSize=captionSize,
+                    leading=1.2*captionSize,
+                    textColor = captionTextColor,
+                    backColor = captionBackColor,
+                    #seems to be getting ignored
+                    spaceBefore=self.captionGap,
+                    alignment=TA_LEFT if captionAlign=='left' else TA_RIGHT if captionAlign=='right' else TA_CENTER,
+                    )
+                #must build paragraph now to get sequencing in synch with rest of story
+                self.captionPara = Paragraph(self.caption, self.captionStyle)
+            else:
+                raise ValueError('Figure caption of type %r is not a string or Paragraph' % type(caption))
 
     def wrap(self, availWidth, availHeight):
         # try to get the caption aligned
         if self.caption:
             self._getCaptionPara()
-            (w, h) = self.captionPara.wrap(self.width, availHeight - self.figureHeight)
-            self.captionHeight = h
+            w, h = self.captionPara.wrap(self.width, availHeight - self.figureHeight)
+            self.captionHeight = h + self.captionGap
             self.height = self.captionHeight + self.figureHeight
             if w>self.width: self.width = w
         else:
             self.height = self.figureHeight
-        self.dx = 0.5 * (availWidth - self.width)
+        if self.hAlign in ('CENTER','CENTRE',TA_CENTER):
+            self.dx = 0.5 * (availWidth - self.width)
+        elif self.hAlign in ('RIGHT',TA_RIGHT):
+            self.dx = availWidth - self.width
+        else:
+            self.dx = 0
         return (self.width, self.height)
 
     def draw(self):
         self.canv.translate(self.dx, 0)
-        if self.caption:
-            self._getCaptionPara()
-            self.drawCaption()
+        if self.caption and self.captionPosition=='bottom':
             self.canv.translate(0, self.captionHeight)
         if self.background:
             self.drawBackground()
         if self.border:
             self.drawBorder()
+        self.canv.saveState()
         self.drawFigure()
+        self.canv.restoreState()
+        if self.caption:
+            if self.captionPosition=='bottom':
+                self.canv.translate(0, -self.captionHeight)
+            else:
+                self.canv.translate(0, self.figureHeight+self.captionGap)
+            self._getCaptionPara()
+            self.drawCaption()
 
     def drawBorder(self):
-        self.canv.rect(0, 0, self.width, self.figureHeight)
+        canv = self.canv
+        border = self.border
+        bc = getattr(border,'color',None)
+        bw = getattr(border,'width',None)
+        bd = getattr(border,'dashArray',None)
+        ss = bc or bw or bd
+        if ss:
+            canv.saveState()
+            if bc: canv.setStrokeColor(bc)
+            if bw: canv.setLineWidth(bw)
+            if bd: canv.setDash(bd)
+        canv.rect(0, 0, self.width, self.figureHeight,fill=0,stroke=1)
+        if ss:
+            canv.restoreState()
 
     def _doBackground(self, color):
         self.canv.saveState()
@@ -173,6 +215,11 @@ class FlexFigure(Figure):
                         spaceBefore=12,
                         spaceAfter=12,
                         captionGap=9,
+                        captionAlign='centre',
+                        captionPosition='top',
+                        scaleFactor=None,
+                        hAlign='CENTER',
+                        border=1,
                         ):
         Figure.__init__(self, width, height, caption,
                         captionFont=captionFont,
@@ -181,11 +228,15 @@ class FlexFigure(Figure):
                         captionTextColor=captionTextColor,
                         spaceBefore = spaceBefore,
                         spaceAfter = spaceAfter,
-                        captionGap=9,
+                        captionGap=captionGap,
+                        captionAlign=captionAlign,
+                        captionPosition=captionPosition,
+                        hAlign=hAlign,
+                        border=border,
                         )
         self.shrinkToFit = shrinkToFit  #if set and wrap is too tight, shrinks
         self.growToFit = growToFit      #if set and wrap is too small, grows
-        self.scaleFactor = None
+        self.scaleFactor = scaleFactor
         self._scaleFactor = None
         self.background = background
 
@@ -216,15 +267,15 @@ class FlexFigure(Figure):
 
 class ImageFigure(FlexFigure):
     """Image with a caption below it"""
-    def __init__(self, filename, caption, background=None):
+    def __init__(self, filename, caption, background=None,scaleFactor=None,hAlign='CENTER',border=None):
         assert os.path.isfile(filename), 'image file %s not found' % filename
         from reportlab.lib.utils import ImageReader
         w, h = ImageReader(filename).getSize()
         self.filename = filename
-        FlexFigure.__init__(self, w, h, caption, background)
+        FlexFigure.__init__(self, w, h, caption, background,scaleFactor=scaleFactor,hAlign=hAlign,border=border)
 
     def drawFigure(self):
-        self.canv.drawInlineImage(self.filename,
+        self.canv.drawImage(self.filename,
                                   0, 0,self.width, self.figureHeight)
 
 class DrawingFigure(FlexFigure):
@@ -244,7 +295,6 @@ class DrawingFigure(FlexFigure):
         self.canv.scale(self._scaleFactor, self._scaleFactor)
         self.drawing.drawOn(self.canv, 0, 0)
 
-
 try:
     from rlextra.pageCatcher.pageCatcher import restoreForms, storeForms, storeFormsInMemory, restoreFormsInMemory
     _hasPageCatcher = 1
@@ -266,7 +316,6 @@ if _hasPageCatcher:
             root, ext = os.path.splitext(filename)
             return '%s_page%d' % (root, pageNo)
 
-
         def needsProcessing(self, pdfFileName, pageNo):
             "returns 1 if no forms or form is older"
             formName = self.getFormName(pdfFileName, pageNo)
@@ -362,7 +411,7 @@ def demo1(canvas):
                     5*inch,     # height
                     showBoundary = 1  # helps us see what's going on
                     )
-    bodyStyle = ParagraphStyle('Body', fontName='Times-Roman', fontSize=24, leading=28, spaceBefore=6)
+    bodyStyle = ParagraphStyle('Body', fontName=_baseFontName, fontSize=24, leading=28, spaceBefore=6)
     para1 = Paragraph('Spam spam spam spam. ' * 5, bodyStyle)
     para2 = Paragraph('Eggs eggs eggs. ' * 5, bodyStyle)
     mydata = [para1, para2]
